/****************************************************************************
** Meta object code from reading C++ file 'qcgtoplevel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kcachegrind-26.08.0/qcachegrind/qcgtoplevel.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qcgtoplevel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11QCGTopLevelE_t {};
} // unnamed namespace

template <> constexpr inline auto QCGTopLevel::qt_create_metaobjectdata<qt_meta_tag_ZN11QCGTopLevelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QCGTopLevel",
        "D-Bus Interface",
        "io.github.qcachegrind",
        "load",
        "",
        "files",
        "addToRecentFiles",
        "add",
        "loadDelayed",
        "file",
        "exportGraph",
        "newWindow",
        "configure",
        "page",
        "about",
        "layoutDuplicate",
        "layoutRemove",
        "layoutNext",
        "layoutPrevious",
        "layoutSave",
        "layoutRestore",
        "updateLayoutActions",
        "updateStatusBar",
        "eventTypeSelected",
        "eventType2Selected",
        "groupTypeSelected",
        "toggleSplitted",
        "toggleSplitDirection",
        "togglePartDock",
        "toggleStackDock",
        "toggleFunctionDock",
        "functionVisibilityChanged",
        "togglePercentage",
        "setPercentage",
        "setAbsoluteCost",
        "setRelativeCost",
        "toggleHideTemplates",
        "toggleExpanded",
        "toggleCycles",
        "recentFilesMenuAboutToShow",
        "recentFilesTriggered",
        "QAction*",
        "sidebarMenuAboutToShow",
        "forwardAboutToShow",
        "forwardTriggered",
        "backAboutToShow",
        "backTriggered",
        "upAboutToShow",
        "upTriggered",
        "primaryAboutToShow",
        "secondaryAboutToShow",
        "groupingAboutToShow",
        "windowListTriggered",
        "windowListAboutToShow",
        "macDockMenuAboutToShow",
        "insertWindowList",
        "QMenu*",
        "reinstallMacDock",
        "setEventType",
        "EventType*",
        "setEventType2",
        "setGroupType",
        "ProfileContext::Type",
        "setGroup",
        "TraceCostItem*",
        "setFunction",
        "TraceFunction*",
        "activePartsChangedSlot",
        "TracePartList",
        "list",
        "partsHideSelectedSlot",
        "partsUnhideAllSlot",
        "setEventTypeDelayed",
        "setEventType2Delayed",
        "setGroupTypeDelayed",
        "setGroupDelayed",
        "setTraceItemDelayed",
        "CostItem*",
        "partsHideSelectedSlotDelayed",
        "partsUnhideAllSlotDelayed",
        "goBack",
        "goForward",
        "goUp",
        "setDirectionDelayed",
        "TraceItemView::Direction",
        "loadFilesDelayed",
        "configChanged",
        "showStatus",
        "msg",
        "progress",
        "showMessage",
        "msec"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'load'
        QtMocHelpers::SlotData<void()>(3, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'load'
        QtMocHelpers::SlotData<void(QStringList, bool)>(3, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 5 }, { QMetaType::Bool, 6 },
        }}),
        // Slot 'load'
        QtMocHelpers::SlotData<void(QStringList)>(3, 4, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QStringList, 5 },
        }}),
        // Slot 'add'
        QtMocHelpers::SlotData<void()>(7, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'add'
        QtMocHelpers::SlotData<void(QStringList)>(7, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 5 },
        }}),
        // Slot 'loadDelayed'
        QtMocHelpers::SlotData<void(QString, bool)>(8, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 }, { QMetaType::Bool, 6 },
        }}),
        // Slot 'loadDelayed'
        QtMocHelpers::SlotData<void(QString)>(8, 4, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 9 },
        }}),
        // Slot 'loadDelayed'
        QtMocHelpers::SlotData<void(QStringList, bool)>(8, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 5 }, { QMetaType::Bool, 6 },
        }}),
        // Slot 'loadDelayed'
        QtMocHelpers::SlotData<void(QStringList)>(8, 4, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QStringList, 5 },
        }}),
        // Slot 'exportGraph'
        QtMocHelpers::SlotData<void()>(10, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'newWindow'
        QtMocHelpers::SlotData<void()>(11, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'configure'
        QtMocHelpers::SlotData<void(QString)>(12, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 },
        }}),
        // Slot 'configure'
        QtMocHelpers::SlotData<void()>(12, 4, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'about'
        QtMocHelpers::SlotData<void()>(14, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'layoutDuplicate'
        QtMocHelpers::SlotData<void()>(15, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'layoutRemove'
        QtMocHelpers::SlotData<void()>(16, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'layoutNext'
        QtMocHelpers::SlotData<void()>(17, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'layoutPrevious'
        QtMocHelpers::SlotData<void()>(18, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'layoutSave'
        QtMocHelpers::SlotData<void()>(19, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'layoutRestore'
        QtMocHelpers::SlotData<void()>(20, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateLayoutActions'
        QtMocHelpers::SlotData<void()>(21, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateStatusBar'
        QtMocHelpers::SlotData<void()>(22, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'eventTypeSelected'
        QtMocHelpers::SlotData<void(const QString &)>(23, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 },
        }}),
        // Slot 'eventType2Selected'
        QtMocHelpers::SlotData<void(const QString &)>(24, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 },
        }}),
        // Slot 'groupTypeSelected'
        QtMocHelpers::SlotData<void(int)>(25, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
        // Slot 'toggleSplitted'
        QtMocHelpers::SlotData<void()>(26, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'toggleSplitDirection'
        QtMocHelpers::SlotData<void()>(27, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'togglePartDock'
        QtMocHelpers::SlotData<void()>(28, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'toggleStackDock'
        QtMocHelpers::SlotData<void()>(29, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'toggleFunctionDock'
        QtMocHelpers::SlotData<void()>(30, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'functionVisibilityChanged'
        QtMocHelpers::SlotData<void(bool)>(31, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 4 },
        }}),
        // Slot 'togglePercentage'
        QtMocHelpers::SlotData<void()>(32, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setPercentage'
        QtMocHelpers::SlotData<void(bool)>(33, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 4 },
        }}),
        // Slot 'setAbsoluteCost'
        QtMocHelpers::SlotData<void()>(34, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setRelativeCost'
        QtMocHelpers::SlotData<void()>(35, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'toggleHideTemplates'
        QtMocHelpers::SlotData<void()>(36, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'toggleExpanded'
        QtMocHelpers::SlotData<void()>(37, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'toggleCycles'
        QtMocHelpers::SlotData<void()>(38, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'recentFilesMenuAboutToShow'
        QtMocHelpers::SlotData<void()>(39, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'recentFilesTriggered'
        QtMocHelpers::SlotData<void(QAction *)>(40, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 41, 4 },
        }}),
        // Slot 'sidebarMenuAboutToShow'
        QtMocHelpers::SlotData<void()>(42, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'forwardAboutToShow'
        QtMocHelpers::SlotData<void()>(43, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'forwardTriggered'
        QtMocHelpers::SlotData<void(QAction *)>(44, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 41, 4 },
        }}),
        // Slot 'backAboutToShow'
        QtMocHelpers::SlotData<void()>(45, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'backTriggered'
        QtMocHelpers::SlotData<void(QAction *)>(46, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 41, 4 },
        }}),
        // Slot 'upAboutToShow'
        QtMocHelpers::SlotData<void()>(47, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'upTriggered'
        QtMocHelpers::SlotData<void(QAction *)>(48, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 41, 4 },
        }}),
        // Slot 'primaryAboutToShow'
        QtMocHelpers::SlotData<void()>(49, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'secondaryAboutToShow'
        QtMocHelpers::SlotData<void()>(50, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'groupingAboutToShow'
        QtMocHelpers::SlotData<void()>(51, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'windowListTriggered'
        QtMocHelpers::SlotData<void(QAction *)>(52, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 41, 4 },
        }}),
        // Slot 'windowListAboutToShow'
        QtMocHelpers::SlotData<void()>(53, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'macDockMenuAboutToShow'
        QtMocHelpers::SlotData<void()>(54, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'insertWindowList'
        QtMocHelpers::SlotData<void(QMenu *)>(55, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 56, 4 },
        }}),
        // Slot 'reinstallMacDock'
        QtMocHelpers::SlotData<void()>(57, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setEventType'
        QtMocHelpers::SlotData<void(EventType *)>(58, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 59, 4 },
        }}),
        // Slot 'setEventType2'
        QtMocHelpers::SlotData<void(EventType *)>(60, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 59, 4 },
        }}),
        // Slot 'setEventType'
        QtMocHelpers::SlotData<void(QString)>(58, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 },
        }}),
        // Slot 'setEventType2'
        QtMocHelpers::SlotData<void(QString)>(60, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 },
        }}),
        // Slot 'setEventType'
        QtMocHelpers::SlotData<void(QAction *)>(58, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 41, 4 },
        }}),
        // Slot 'setEventType2'
        QtMocHelpers::SlotData<void(QAction *)>(60, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 41, 4 },
        }}),
        // Slot 'setGroupType'
        QtMocHelpers::SlotData<bool(ProfileContext::Type)>(61, 4, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 62, 4 },
        }}),
        // Slot 'setGroupType'
        QtMocHelpers::SlotData<bool(QString)>(61, 4, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 4 },
        }}),
        // Slot 'setGroup'
        QtMocHelpers::SlotData<bool(TraceCostItem *)>(63, 4, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 64, 4 },
        }}),
        // Slot 'setGroup'
        QtMocHelpers::SlotData<bool(QString)>(63, 4, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 4 },
        }}),
        // Slot 'setFunction'
        QtMocHelpers::SlotData<bool(TraceFunction *)>(65, 4, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 66, 4 },
        }}),
        // Slot 'setFunction'
        QtMocHelpers::SlotData<bool(QString)>(65, 4, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 4 },
        }}),
        // Slot 'activePartsChangedSlot'
        QtMocHelpers::SlotData<void(const TracePartList &)>(67, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 68, 69 },
        }}),
        // Slot 'partsHideSelectedSlot'
        QtMocHelpers::SlotData<void()>(70, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'partsUnhideAllSlot'
        QtMocHelpers::SlotData<void()>(71, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setEventTypeDelayed'
        QtMocHelpers::SlotData<void(EventType *)>(72, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 59, 4 },
        }}),
        // Slot 'setEventType2Delayed'
        QtMocHelpers::SlotData<void(EventType *)>(73, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 59, 4 },
        }}),
        // Slot 'setGroupTypeDelayed'
        QtMocHelpers::SlotData<void(ProfileContext::Type)>(74, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 62, 4 },
        }}),
        // Slot 'setGroupDelayed'
        QtMocHelpers::SlotData<void(TraceCostItem *)>(75, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 64, 4 },
        }}),
        // Slot 'setTraceItemDelayed'
        QtMocHelpers::SlotData<void(CostItem *)>(76, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 77, 4 },
        }}),
        // Slot 'partsHideSelectedSlotDelayed'
        QtMocHelpers::SlotData<void()>(78, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'partsUnhideAllSlotDelayed'
        QtMocHelpers::SlotData<void()>(79, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'goBack'
        QtMocHelpers::SlotData<void()>(80, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'goForward'
        QtMocHelpers::SlotData<void()>(81, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'goUp'
        QtMocHelpers::SlotData<void()>(82, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setDirectionDelayed'
        QtMocHelpers::SlotData<void(TraceItemView::Direction)>(83, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 84, 4 },
        }}),
        // Slot 'setEventTypeDelayed'
        QtMocHelpers::SlotData<void()>(72, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setEventType2Delayed'
        QtMocHelpers::SlotData<void()>(73, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setGroupTypeDelayed'
        QtMocHelpers::SlotData<void()>(74, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setGroupDelayed'
        QtMocHelpers::SlotData<void()>(75, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setTraceItemDelayed'
        QtMocHelpers::SlotData<void()>(76, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'loadFilesDelayed'
        QtMocHelpers::SlotData<void()>(85, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setDirectionDelayed'
        QtMocHelpers::SlotData<void()>(83, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'configChanged'
        QtMocHelpers::SlotData<void()>(86, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showStatus'
        QtMocHelpers::SlotData<void(const QString &, int)>(87, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 88 }, { QMetaType::Int, 89 },
        }}),
        // Slot 'showMessage'
        QtMocHelpers::SlotData<void(const QString &, int)>(90, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 }, { QMetaType::Int, 91 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<QCGTopLevel, qt_meta_tag_ZN11QCGTopLevelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QCGTopLevel::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11QCGTopLevelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11QCGTopLevelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11QCGTopLevelE_t>.metaTypes,
    nullptr
} };

void QCGTopLevel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QCGTopLevel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->load(); break;
        case 1: _t->load((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 2: _t->load((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 3: _t->add(); break;
        case 4: _t->add((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 5: _t->loadDelayed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 6: _t->loadDelayed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->loadDelayed((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 8: _t->loadDelayed((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 9: _t->exportGraph(); break;
        case 10: _t->newWindow(); break;
        case 11: _t->configure((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 12: _t->configure(); break;
        case 13: _t->about(); break;
        case 14: _t->layoutDuplicate(); break;
        case 15: _t->layoutRemove(); break;
        case 16: _t->layoutNext(); break;
        case 17: _t->layoutPrevious(); break;
        case 18: _t->layoutSave(); break;
        case 19: _t->layoutRestore(); break;
        case 20: _t->updateLayoutActions(); break;
        case 21: _t->updateStatusBar(); break;
        case 22: _t->eventTypeSelected((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 23: _t->eventType2Selected((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 24: _t->groupTypeSelected((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 25: _t->toggleSplitted(); break;
        case 26: _t->toggleSplitDirection(); break;
        case 27: _t->togglePartDock(); break;
        case 28: _t->toggleStackDock(); break;
        case 29: _t->toggleFunctionDock(); break;
        case 30: _t->functionVisibilityChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 31: _t->togglePercentage(); break;
        case 32: _t->setPercentage((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 33: _t->setAbsoluteCost(); break;
        case 34: _t->setRelativeCost(); break;
        case 35: _t->toggleHideTemplates(); break;
        case 36: _t->toggleExpanded(); break;
        case 37: _t->toggleCycles(); break;
        case 38: _t->recentFilesMenuAboutToShow(); break;
        case 39: _t->recentFilesTriggered((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 40: _t->sidebarMenuAboutToShow(); break;
        case 41: _t->forwardAboutToShow(); break;
        case 42: _t->forwardTriggered((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 43: _t->backAboutToShow(); break;
        case 44: _t->backTriggered((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 45: _t->upAboutToShow(); break;
        case 46: _t->upTriggered((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 47: _t->primaryAboutToShow(); break;
        case 48: _t->secondaryAboutToShow(); break;
        case 49: _t->groupingAboutToShow(); break;
        case 50: _t->windowListTriggered((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 51: _t->windowListAboutToShow(); break;
        case 52: _t->macDockMenuAboutToShow(); break;
        case 53: _t->insertWindowList((*reinterpret_cast<std::add_pointer_t<QMenu*>>(_a[1]))); break;
        case 54: _t->reinstallMacDock(); break;
        case 55: _t->setEventType((*reinterpret_cast<std::add_pointer_t<EventType*>>(_a[1]))); break;
        case 56: _t->setEventType2((*reinterpret_cast<std::add_pointer_t<EventType*>>(_a[1]))); break;
        case 57: _t->setEventType((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 58: _t->setEventType2((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 59: _t->setEventType((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 60: _t->setEventType2((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 61: { bool _r = _t->setGroupType((*reinterpret_cast<std::add_pointer_t<ProfileContext::Type>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 62: { bool _r = _t->setGroupType((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 63: { bool _r = _t->setGroup((*reinterpret_cast<std::add_pointer_t<TraceCostItem*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 64: { bool _r = _t->setGroup((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 65: { bool _r = _t->setFunction((*reinterpret_cast<std::add_pointer_t<TraceFunction*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 66: { bool _r = _t->setFunction((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 67: _t->activePartsChangedSlot((*reinterpret_cast<std::add_pointer_t<TracePartList>>(_a[1]))); break;
        case 68: _t->partsHideSelectedSlot(); break;
        case 69: _t->partsUnhideAllSlot(); break;
        case 70: _t->setEventTypeDelayed((*reinterpret_cast<std::add_pointer_t<EventType*>>(_a[1]))); break;
        case 71: _t->setEventType2Delayed((*reinterpret_cast<std::add_pointer_t<EventType*>>(_a[1]))); break;
        case 72: _t->setGroupTypeDelayed((*reinterpret_cast<std::add_pointer_t<ProfileContext::Type>>(_a[1]))); break;
        case 73: _t->setGroupDelayed((*reinterpret_cast<std::add_pointer_t<TraceCostItem*>>(_a[1]))); break;
        case 74: _t->setTraceItemDelayed((*reinterpret_cast<std::add_pointer_t<CostItem*>>(_a[1]))); break;
        case 75: _t->partsHideSelectedSlotDelayed(); break;
        case 76: _t->partsUnhideAllSlotDelayed(); break;
        case 77: _t->goBack(); break;
        case 78: _t->goForward(); break;
        case 79: _t->goUp(); break;
        case 80: _t->setDirectionDelayed((*reinterpret_cast<std::add_pointer_t<TraceItemView::Direction>>(_a[1]))); break;
        case 81: _t->setEventTypeDelayed(); break;
        case 82: _t->setEventType2Delayed(); break;
        case 83: _t->setGroupTypeDelayed(); break;
        case 84: _t->setGroupDelayed(); break;
        case 85: _t->setTraceItemDelayed(); break;
        case 86: _t->loadFilesDelayed(); break;
        case 87: _t->setDirectionDelayed(); break;
        case 88: _t->configChanged(); break;
        case 89: _t->showStatus((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 90: _t->showMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 39:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        case 42:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        case 44:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        case 46:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        case 50:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        case 59:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        case 60:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        }
    }
}

const QMetaObject *QCGTopLevel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QCGTopLevel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11QCGTopLevelE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Logger"))
        return static_cast< Logger*>(this);
    if (!strcmp(_clname, "TopLevelBase"))
        return static_cast< TopLevelBase*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int QCGTopLevel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 91)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 91;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 91)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 91;
    }
    return _id;
}
QT_WARNING_POP
