/****************************************************************************
** Meta object code from reading C++ file 'stackselection.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kcachegrind-26.08.0/libviews/stackselection.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'stackselection.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14StackSelectionE_t {};
} // unnamed namespace

template <> constexpr inline auto StackSelection::qt_create_metaobjectdata<qt_meta_tag_ZN14StackSelectionE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "StackSelection",
        "functionSelected",
        "",
        "CostItem*",
        "setFunction",
        "TraceFunction*",
        "setEventType",
        "EventType*",
        "setEventType2",
        "setGroupType",
        "ProfileContext::Type",
        "stackSelected",
        "QTreeWidgetItem*",
        "browserBack",
        "browserForward",
        "browserUp",
        "browserDown",
        "refresh",
        "rebuildStackList"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'functionSelected'
        QtMocHelpers::SignalData<void(CostItem *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Slot 'setFunction'
        QtMocHelpers::SlotData<void(TraceFunction *)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 2 },
        }}),
        // Slot 'setEventType'
        QtMocHelpers::SlotData<void(EventType *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 2 },
        }}),
        // Slot 'setEventType2'
        QtMocHelpers::SlotData<void(EventType *)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 2 },
        }}),
        // Slot 'setGroupType'
        QtMocHelpers::SlotData<void(ProfileContext::Type)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 2 },
        }}),
        // Slot 'stackSelected'
        QtMocHelpers::SlotData<void(QTreeWidgetItem *, QTreeWidgetItem *)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 2 }, { 0x80000000 | 12, 2 },
        }}),
        // Slot 'browserBack'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'browserForward'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'browserUp'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'browserDown'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'refresh'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'rebuildStackList'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<StackSelection, qt_meta_tag_ZN14StackSelectionE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject StackSelection::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14StackSelectionE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14StackSelectionE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14StackSelectionE_t>.metaTypes,
    nullptr
} };

void StackSelection::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<StackSelection *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->functionSelected((*reinterpret_cast<std::add_pointer_t<CostItem*>>(_a[1]))); break;
        case 1: _t->setFunction((*reinterpret_cast<std::add_pointer_t<TraceFunction*>>(_a[1]))); break;
        case 2: _t->setEventType((*reinterpret_cast<std::add_pointer_t<EventType*>>(_a[1]))); break;
        case 3: _t->setEventType2((*reinterpret_cast<std::add_pointer_t<EventType*>>(_a[1]))); break;
        case 4: _t->setGroupType((*reinterpret_cast<std::add_pointer_t<ProfileContext::Type>>(_a[1]))); break;
        case 5: _t->stackSelected((*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[2]))); break;
        case 6: _t->browserBack(); break;
        case 7: _t->browserForward(); break;
        case 8: _t->browserUp(); break;
        case 9: _t->browserDown(); break;
        case 10: _t->refresh(); break;
        case 11: _t->rebuildStackList(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (StackSelection::*)(CostItem * )>(_a, &StackSelection::functionSelected, 0))
            return;
    }
}

const QMetaObject *StackSelection::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StackSelection::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14StackSelectionE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int StackSelection::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void StackSelection::functionSelected(CostItem * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
QT_WARNING_POP
