/********************************************************************************
** Form generated from reading UI file 'configdlgbase.ui'
**
** Created by: Qt User Interface Compiler version 6.11.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CONFIGDLGBASE_H
#define UI_CONFIGDLGBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kcolorbutton.h"

QT_BEGIN_NAMESPACE

class Ui_ConfigDlgBase
{
public:
    QVBoxLayout *verticalLayout;
    QTabWidget *tabWidget2;
    QWidget *tab1;
    QGridLayout *gridLayout_2;
    QGridLayout *gridLayout;
    QGridLayout *gridLayout1;
    QLabel *TextLabel5;
    QLabel *TextLabel3;
    QSpacerItem *Spacer6_2_2_2;
    QLabel *TextLabel2;
    QLabel *TextLabel2_2;
    QLabel *TextLabel4_3;
    QSpacerItem *horizontalSpacer_2;
    QSpinBox *maxListEdit;
    QSpinBox *symbolCount;
    QSpinBox *symbolLength;
    QSpinBox *precisionEdit;
    QLabel *TextLabel1;
    QGridLayout *gridLayout2;
    QCheckBox *fileCheck;
    QLabel *TextLabel4;
    QLabel *TextLabel4_2_2;
    KColorButton *fileColor;
    QCheckBox *classCheck;
    KColorButton *objectColor;
    QCheckBox *objectCheck;
    QLabel *TextLabel4_2;
    KColorButton *classColor;
    QSpacerItem *horizontalSpacer;
    QComboBox *objectCombo;
    QComboBox *classCombo;
    QComboBox *fileCombo;
    QSpacerItem *verticalSpacer;
    QWidget *tab2;
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout;
    QLabel *TextLabel4_3_2;
    QSpinBox *contextEdit;
    QLabel *TextLabel1_2;
    QGridLayout *gridLayout3;
    QVBoxLayout *vboxLayout1;
    QPushButton *addDirButton;
    QPushButton *deleteDirButton;
    QSpacerItem *Spacer5;
    QTreeWidget *dirList;
    QFrame *line;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *ConfigDlgBase)
    {
        if (ConfigDlgBase->objectName().isEmpty())
            ConfigDlgBase->setObjectName("ConfigDlgBase");
        ConfigDlgBase->resize(435, 439);
        verticalLayout = new QVBoxLayout(ConfigDlgBase);
        verticalLayout->setObjectName("verticalLayout");
        tabWidget2 = new QTabWidget(ConfigDlgBase);
        tabWidget2->setObjectName("tabWidget2");
        tab1 = new QWidget();
        tab1->setObjectName("tab1");
        gridLayout_2 = new QGridLayout(tab1);
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        gridLayout1 = new QGridLayout();
        gridLayout1->setObjectName("gridLayout1");
        TextLabel5 = new QLabel(tab1);
        TextLabel5->setObjectName("TextLabel5");
        TextLabel5->setWordWrap(false);

        gridLayout1->addWidget(TextLabel5, 0, 0, 1, 2);

        TextLabel3 = new QLabel(tab1);
        TextLabel3->setObjectName("TextLabel3");
        TextLabel3->setWordWrap(false);

        gridLayout1->addWidget(TextLabel3, 1, 0, 1, 4);

        Spacer6_2_2_2 = new QSpacerItem(16, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        gridLayout1->addItem(Spacer6_2_2_2, 2, 0, 1, 1);

        TextLabel2 = new QLabel(tab1);
        TextLabel2->setObjectName("TextLabel2");
        TextLabel2->setWordWrap(true);

        gridLayout1->addWidget(TextLabel2, 2, 1, 1, 1);

        TextLabel2_2 = new QLabel(tab1);
        TextLabel2_2->setObjectName("TextLabel2_2");
        TextLabel2_2->setWordWrap(true);

        gridLayout1->addWidget(TextLabel2_2, 3, 1, 1, 1);

        TextLabel4_3 = new QLabel(tab1);
        TextLabel4_3->setObjectName("TextLabel4_3");
        TextLabel4_3->setWordWrap(false);

        gridLayout1->addWidget(TextLabel4_3, 4, 0, 1, 2);

        horizontalSpacer_2 = new QSpacerItem(0, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout1->addItem(horizontalSpacer_2, 0, 3, 1, 1);

        maxListEdit = new QSpinBox(tab1);
        maxListEdit->setObjectName("maxListEdit");
        maxListEdit->setMaximum(499);

        gridLayout1->addWidget(maxListEdit, 0, 2, 1, 1);

        symbolCount = new QSpinBox(tab1);
        symbolCount->setObjectName("symbolCount");
        symbolCount->setMaximum(999);

        gridLayout1->addWidget(symbolCount, 2, 2, 1, 1);

        symbolLength = new QSpinBox(tab1);
        symbolLength->setObjectName("symbolLength");
        symbolLength->setMaximum(999);

        gridLayout1->addWidget(symbolLength, 3, 2, 1, 1);

        precisionEdit = new QSpinBox(tab1);
        precisionEdit->setObjectName("precisionEdit");
        precisionEdit->setMaximum(10);

        gridLayout1->addWidget(precisionEdit, 4, 2, 1, 1);


        gridLayout->addLayout(gridLayout1, 0, 0, 1, 1);

        TextLabel1 = new QLabel(tab1);
        TextLabel1->setObjectName("TextLabel1");
        QFont font;
        font.setBold(true);
        TextLabel1->setFont(font);
        TextLabel1->setFrameShape(QFrame::NoFrame);
        TextLabel1->setFrameShadow(QFrame::Plain);
        TextLabel1->setWordWrap(false);

        gridLayout->addWidget(TextLabel1, 1, 0, 1, 1);

        gridLayout2 = new QGridLayout();
        gridLayout2->setSpacing(6);
        gridLayout2->setObjectName("gridLayout2");
        gridLayout2->setContentsMargins(0, 0, 0, 0);
        fileCheck = new QCheckBox(tab1);
        fileCheck->setObjectName("fileCheck");

        gridLayout2->addWidget(fileCheck, 2, 2, 1, 1);

        TextLabel4 = new QLabel(tab1);
        TextLabel4->setObjectName("TextLabel4");
        TextLabel4->setWordWrap(false);

        gridLayout2->addWidget(TextLabel4, 0, 0, 1, 1);

        TextLabel4_2_2 = new QLabel(tab1);
        TextLabel4_2_2->setObjectName("TextLabel4_2_2");
        TextLabel4_2_2->setWordWrap(false);

        gridLayout2->addWidget(TextLabel4_2_2, 1, 0, 1, 1);

        fileColor = new KColorButton(tab1);
        fileColor->setObjectName("fileColor");

        gridLayout2->addWidget(fileColor, 2, 3, 1, 1);

        classCheck = new QCheckBox(tab1);
        classCheck->setObjectName("classCheck");

        gridLayout2->addWidget(classCheck, 1, 2, 1, 1);

        objectColor = new KColorButton(tab1);
        objectColor->setObjectName("objectColor");

        gridLayout2->addWidget(objectColor, 0, 3, 1, 1);

        objectCheck = new QCheckBox(tab1);
        objectCheck->setObjectName("objectCheck");

        gridLayout2->addWidget(objectCheck, 0, 2, 1, 1);

        TextLabel4_2 = new QLabel(tab1);
        TextLabel4_2->setObjectName("TextLabel4_2");
        TextLabel4_2->setWordWrap(false);

        gridLayout2->addWidget(TextLabel4_2, 2, 0, 1, 1);

        classColor = new KColorButton(tab1);
        classColor->setObjectName("classColor");

        gridLayout2->addWidget(classColor, 1, 3, 1, 1);

        horizontalSpacer = new QSpacerItem(0, 20, QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Minimum);

        gridLayout2->addItem(horizontalSpacer, 0, 4, 1, 1);

        objectCombo = new QComboBox(tab1);
        objectCombo->setObjectName("objectCombo");
        objectCombo->setMinimumSize(QSize(150, 0));
        objectCombo->setEditable(true);

        gridLayout2->addWidget(objectCombo, 0, 1, 1, 1);

        classCombo = new QComboBox(tab1);
        classCombo->setObjectName("classCombo");
        classCombo->setEditable(true);

        gridLayout2->addWidget(classCombo, 1, 1, 1, 1);

        fileCombo = new QComboBox(tab1);
        fileCombo->setObjectName("fileCombo");
        fileCombo->setEditable(true);

        gridLayout2->addWidget(fileCombo, 2, 1, 1, 1);


        gridLayout->addLayout(gridLayout2, 2, 0, 1, 1);


        gridLayout_2->addLayout(gridLayout, 0, 0, 1, 1);

        verticalSpacer = new QSpacerItem(20, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::MinimumExpanding);

        gridLayout_2->addItem(verticalSpacer, 1, 0, 1, 1);

        tabWidget2->addTab(tab1, QString());
        tab2 = new QWidget();
        tab2->setObjectName("tab2");
        vboxLayout = new QVBoxLayout(tab2);
        vboxLayout->setObjectName("vboxLayout");
        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName("hboxLayout");
        TextLabel4_3_2 = new QLabel(tab2);
        TextLabel4_3_2->setObjectName("TextLabel4_3_2");
        TextLabel4_3_2->setWordWrap(false);

        hboxLayout->addWidget(TextLabel4_3_2);

        contextEdit = new QSpinBox(tab2);
        contextEdit->setObjectName("contextEdit");

        hboxLayout->addWidget(contextEdit);


        vboxLayout->addLayout(hboxLayout);

        TextLabel1_2 = new QLabel(tab2);
        TextLabel1_2->setObjectName("TextLabel1_2");
        TextLabel1_2->setFont(font);
        TextLabel1_2->setWordWrap(false);

        vboxLayout->addWidget(TextLabel1_2);

        gridLayout3 = new QGridLayout();
        gridLayout3->setObjectName("gridLayout3");
        vboxLayout1 = new QVBoxLayout();
        vboxLayout1->setObjectName("vboxLayout1");
        addDirButton = new QPushButton(tab2);
        addDirButton->setObjectName("addDirButton");

        vboxLayout1->addWidget(addDirButton);

        deleteDirButton = new QPushButton(tab2);
        deleteDirButton->setObjectName("deleteDirButton");

        vboxLayout1->addWidget(deleteDirButton);

        Spacer5 = new QSpacerItem(16, 49, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout1->addItem(Spacer5);


        gridLayout3->addLayout(vboxLayout1, 0, 1, 1, 1);

        dirList = new QTreeWidget(tab2);
        dirList->setObjectName("dirList");
        dirList->setMinimumSize(QSize(300, 0));
        dirList->setEditTriggers(QAbstractItemView::NoEditTriggers);
        dirList->setProperty("showDropIndicator", QVariant(false));

        gridLayout3->addWidget(dirList, 0, 0, 1, 1);


        vboxLayout->addLayout(gridLayout3);

        tabWidget2->addTab(tab2, QString());

        verticalLayout->addWidget(tabWidget2);

        line = new QFrame(ConfigDlgBase);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout->addWidget(line);

        buttonBox = new QDialogButtonBox(ConfigDlgBase);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);

        QWidget::setTabOrder(tabWidget2, objectCombo);
        QWidget::setTabOrder(objectCombo, objectCheck);
        QWidget::setTabOrder(objectCheck, objectColor);
        QWidget::setTabOrder(objectColor, classCombo);
        QWidget::setTabOrder(classCombo, classCheck);
        QWidget::setTabOrder(classCheck, classColor);
        QWidget::setTabOrder(classColor, fileCombo);
        QWidget::setTabOrder(fileCombo, fileCheck);
        QWidget::setTabOrder(fileCheck, fileColor);
        QWidget::setTabOrder(fileColor, buttonBox);
        QWidget::setTabOrder(buttonBox, dirList);

        retranslateUi(ConfigDlgBase);
        QObject::connect(classCheck, SIGNAL(toggled(bool)), classColor, SLOT(setDisabled(bool)));
        QObject::connect(fileCheck, SIGNAL(toggled(bool)), fileColor, SLOT(setDisabled(bool)));
        QObject::connect(objectCheck, SIGNAL(toggled(bool)), objectColor, SLOT(setDisabled(bool)));

        tabWidget2->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(ConfigDlgBase);
    } // setupUi

    void retranslateUi(QDialog *ConfigDlgBase)
    {
        ConfigDlgBase->setWindowTitle(tr2i18n("Configuration", nullptr));
        TextLabel5->setText(tr2i18n("Maximum number of items in lists:", nullptr));
        TextLabel3->setText(tr2i18n("Truncate symbols in tooltips and context menus", nullptr));
        TextLabel2->setText(tr2i18n("when more than:", nullptr));
        TextLabel2_2->setText(tr2i18n("when longer than:", nullptr));
        TextLabel4_3->setText(tr2i18n("Precision of percentage values:", nullptr));
#if QT_CONFIG(tooltip)
        maxListEdit->setToolTip(tr2i18n("The Maximum Number of List Items should be below 500.", nullptr));
#endif // QT_CONFIG(tooltip)
        TextLabel1->setText(tr2i18n("Cost Item Colors", nullptr));
        fileCheck->setText(tr2i18n("Automatic", nullptr));
        TextLabel4->setText(tr2i18n("Object:", nullptr));
        TextLabel4_2_2->setText(tr2i18n("Class:", nullptr));
        fileColor->setText(QString());
        classCheck->setText(tr2i18n("Automatic", nullptr));
        objectColor->setText(QString());
        objectCheck->setText(tr2i18n("Automatic", nullptr));
        TextLabel4_2->setText(tr2i18n("File:", nullptr));
        classColor->setText(QString());
        tabWidget2->setTabText(tabWidget2->indexOf(tab1), tr2i18n("General", nullptr));
        TextLabel4_3_2->setText(tr2i18n("Context lines in annotations:", nullptr));
        TextLabel1_2->setText(tr2i18n("Source Folders", nullptr));
        addDirButton->setText(tr2i18n("Add", nullptr));
        deleteDirButton->setText(tr2i18n("Delete", nullptr));
        QTreeWidgetItem *___qtreewidgetitem = dirList->headerItem();
        ___qtreewidgetitem->setText(0, tr2i18n("Object / Related Source Base", nullptr));
        tabWidget2->setTabText(tabWidget2->indexOf(tab2), tr2i18n("Annotations", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ConfigDlgBase: public Ui_ConfigDlgBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // CONFIGDLGBASE_H

