/****************************************************************************
** Resource object code
**
** Created by: The Resource Compiler for Qt version 6.11.2
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#ifdef _MSC_VER
// disable informational message "function ... selected for automatic inline expansion"
#pragma warning (disable: 4711)
#endif

static const unsigned char qt_resource_data[] = {
  // kcachegrindui.rc
  0x0,0x0,0x2,0x7,
  0x28,
  0xb5,0x2f,0xfd,0x60,0xef,0x5,0xed,0xf,0x0,0x16,0x16,0x42,0x1d,0x50,0x55,0x1e,
  0xf,0x49,0x4a,0xde,0xc4,0xd8,0x44,0x5a,0xd8,0xf2,0xe9,0x37,0x4,0x15,0xf5,0x61,
  0x1a,0xbf,0xc4,0xee,0x3d,0xb2,0xcc,0x3c,0x67,0x6a,0x3e,0x0,0x38,0x0,0x36,0x0,
  0x73,0x5a,0x41,0xf4,0x1c,0x7e,0x28,0x5a,0x1c,0xe1,0xc5,0x57,0x51,0x25,0x43,0x20,
  0x65,0x58,0x8,0xd9,0x90,0x3a,0x6b,0xd4,0x72,0x12,0xfa,0x63,0x4c,0xdb,0x96,0x59,
  0x8,0x9e,0x56,0x42,0x80,0x68,0xb9,0x77,0x83,0xf7,0x9d,0x99,0xa5,0x20,0x8d,0x89,
  0x54,0x91,0x14,0xac,0x4f,0x35,0x24,0x95,0xc9,0x22,0xc2,0x8,0x40,0x1,0xd1,0xfb,
  0xd7,0x77,0xb2,0x34,0x6b,0xfb,0x2b,0x3c,0x8b,0x67,0xe7,0xd9,0xd3,0x6e,0x8c,0xb2,
  0x61,0xa8,0x79,0xa0,0xf7,0x9e,0x1c,0x9e,0x53,0x75,0xef,0xac,0x61,0xdf,0x8e,0xec,
  0xcf,0xe2,0xe7,0x3c,0xb1,0x5e,0x8,0x27,0x79,0xce,0x55,0xcb,0xb9,0x65,0xb2,0xe1,
  0xb7,0xc4,0x3,0x39,0xe,0x1,0x8b,0x3e,0x9,0xff,0x99,0x52,0xfe,0x8d,0x73,0xe2,
  0x67,0xec,0x20,0x86,0x72,0x6c,0x1d,0xc7,0x4e,0xee,0xdb,0x97,0xdc,0x9c,0xa7,0x43,
  0xc7,0x60,0x9c,0x74,0xa9,0xf7,0x53,0x4d,0x4f,0xec,0xfc,0xdb,0x60,0x9c,0xd5,0xd1,
  0xda,0x50,0xc3,0x3e,0x39,0x2c,0xd9,0x83,0x7e,0xed,0xce,0x6,0x80,0x8,0xd6,0xf7,
  0x40,0x0,0x25,0x3d,0x8d,0x66,0x75,0xbe,0x1b,0xef,0xcb,0x5a,0x58,0x3f,0xc9,0xb3,
  0x27,0xa,0x42,0x78,0xce,0x90,0x96,0x6a,0x49,0xb8,0x53,0xec,0x9e,0x70,0x84,0x97,
  0xf6,0x4c,0xdb,0xa7,0xcf,0xca,0xb4,0x76,0x8c,0xb2,0x83,0x92,0xaf,0x25,0x76,0x3f,
  0xdb,0x34,0x86,0x6,0x66,0xa0,0xd1,0x9d,0x12,0x9c,0x22,0x21,0x26,0x5,0x29,0x2c,
  0x7,0x20,0x46,0x60,0x94,0x44,0x54,0x1e,0xf7,0x39,0x60,0xab,0x8c,0xcc,0x6a,0xf,
  0x58,0x94,0x19,0x46,0x22,0x40,0x32,0x4f,0x47,0xe6,0x7a,0x40,0x46,0x72,0xb4,0xd2,
  0x88,0x8d,0x44,0x8f,0xc4,0x39,0xf2,0x8f,0x9,0xae,0x7b,0x1e,0x5f,0x6c,0x69,0xc4,
  0x8b,0x9e,0x29,0xe8,0x1,0x36,0xf9,0x39,0x42,0x56,0xc6,0x21,0x4,0x81,0x30,0x4c,
  0x80,0x1c,0x50,0x4d,0x46,0xba,0x4c,0x2f,0x0,0x2,0x8,0x4c,0xa6,0xbd,0x69,0x1d,
  0xb,0xa2,0xc9,0xc5,0x86,0xf3,0xd0,0xa0,0xd4,0xd1,0x3a,0xdf,0x5c,0xdb,0x6,0x10,
  0xbf,0xda,0x8e,0x6f,0xc3,0x19,0x35,0x8e,0x2,0xd9,0x68,0x16,0x36,0x36,0x5b,0xb1,
  0x64,0x52,0xd4,0x40,0x6d,0xc5,0xe4,0xf8,0x57,0x1b,0x1a,0x8e,0x53,0xc0,0xe5,0x17,
  0xb6,0xf8,0xd2,0xe7,0x97,0xa0,0x5b,0xf7,0x1,0xbe,0xf0,0x47,0x87,0x11,0xee,0xd9,
  0x79,0x35,0x32,0xdd,0xd9,0xe3,0x1e,0x63,0xa2,0x14,0x27,0x29,0x52,0xc4,0xcc,0x0,
  0x0,0xcf,0x54,0xdd,0xf8,0x9a,0xe1,0x72,0x7d,0x2d,0x9e,0x2a,0x19,0x1c,0xcc,0x7a,
  0xf9,0xd0,0xa3,0xbd,0x18,0x87,0xd1,0xc4,0xe1,0x95,0x2,0x70,0x1c,0x91,0x89,0xb2,
  0x55,0x60,0xc7,0xc1,0x70,0x66,0x3b,0x98,0x24,0x2e,0x94,0x12,0x49,0x0,0x8a,0x19,
  0xbf,0x0,0xae,0x50,0x83,0x80,0x41,0x80,0x6d,0xe6,0xe7,0xa1,0xc,0x80,0x86,0xa,
  0xf5,0x8,0xf8,0x9,0xc0,0x6a,
  
};

static const unsigned char qt_resource_name[] = {
  // kxmlgui5
  0x0,0x8,
  0xf,0x42,0xe5,0x85,
  0x0,0x6b,
  0x0,0x78,0x0,0x6d,0x0,0x6c,0x0,0x67,0x0,0x75,0x0,0x69,0x0,0x35,
    // kcachegrind
  0x0,0xb,
  0xe,0x5d,0xa3,0x64,
  0x0,0x6b,
  0x0,0x63,0x0,0x61,0x0,0x63,0x0,0x68,0x0,0x65,0x0,0x67,0x0,0x72,0x0,0x69,0x0,0x6e,0x0,0x64,
    // kcachegrindui.rc
  0x0,0x10,
  0x7,0x70,0x31,0xe3,
  0x0,0x6b,
  0x0,0x63,0x0,0x61,0x0,0x63,0x0,0x68,0x0,0x65,0x0,0x67,0x0,0x72,0x0,0x69,0x0,0x6e,0x0,0x64,0x0,0x75,0x0,0x69,0x0,0x2e,0x0,0x72,0x0,0x63,
  
};

static const unsigned char qt_resource_struct[] = {
  // :
  0x0,0x0,0x0,0x0,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x1,
0x0,0x0,0x1,0xa0,0x1a,0xf5,0x61,0x60,
  // :/kxmlgui5
  0x0,0x0,0x0,0x0,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x2,
0x0,0x0,0x1,0xa0,0x1a,0xf5,0x61,0x60,
  // :/kxmlgui5/kcachegrind
  0x0,0x0,0x0,0x16,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x3,
0x0,0x0,0x1,0xa0,0x1a,0xf5,0x61,0x60,
  // :/kxmlgui5/kcachegrind/kcachegrindui.rc
  0x0,0x0,0x0,0x32,0x0,0x4,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x0,
0x0,0x0,0x1,0xa0,0x1a,0xf5,0x61,0x60,

};

#ifdef QT_NAMESPACE
#  define QT_RCC_PREPEND_NAMESPACE(name) ::QT_NAMESPACE::name
#  define QT_RCC_MANGLE_NAMESPACE0(x) x
#  define QT_RCC_MANGLE_NAMESPACE1(a, b) a##_##b
#  define QT_RCC_MANGLE_NAMESPACE2(a, b) QT_RCC_MANGLE_NAMESPACE1(a,b)
#  define QT_RCC_MANGLE_NAMESPACE(name) QT_RCC_MANGLE_NAMESPACE2( \
        QT_RCC_MANGLE_NAMESPACE0(name), QT_RCC_MANGLE_NAMESPACE0(QT_NAMESPACE))
#else
#   define QT_RCC_PREPEND_NAMESPACE(name) name
#   define QT_RCC_MANGLE_NAMESPACE(name) name
#endif

#if defined(QT_INLINE_NAMESPACE)
inline namespace QT_NAMESPACE {
#elif defined(QT_NAMESPACE)
namespace QT_NAMESPACE {
#endif

bool qRegisterResourceData(int, const unsigned char *, const unsigned char *, const unsigned char *);
bool qUnregisterResourceData(int, const unsigned char *, const unsigned char *, const unsigned char *);

#if defined(__ELF__) || defined(__APPLE__)
static inline unsigned char qResourceFeatureZstd()
{
    extern const unsigned char qt_resourceFeatureZstd;
    return qt_resourceFeatureZstd;
}
#else
unsigned char qResourceFeatureZstd();
#endif

#ifdef QT_NAMESPACE
}
#endif

int QT_RCC_MANGLE_NAMESPACE(qInitResources_kcachegrind)();
int QT_RCC_MANGLE_NAMESPACE(qInitResources_kcachegrind)()
{
    int version = 3;
    QT_RCC_PREPEND_NAMESPACE(qRegisterResourceData)
        (version, qt_resource_struct, qt_resource_name, qt_resource_data);
    return 1;
}

int QT_RCC_MANGLE_NAMESPACE(qCleanupResources_kcachegrind)();
int QT_RCC_MANGLE_NAMESPACE(qCleanupResources_kcachegrind)()
{
    int version = 3;
    version += QT_RCC_PREPEND_NAMESPACE(qResourceFeatureZstd());
    QT_RCC_PREPEND_NAMESPACE(qUnregisterResourceData)
       (version, qt_resource_struct, qt_resource_name, qt_resource_data);
    return 1;
}

#ifdef __clang__
#   pragma clang diagnostic push
#   pragma clang diagnostic ignored "-Wexit-time-destructors"
#endif

namespace {
   struct initializer {
       initializer() { QT_RCC_MANGLE_NAMESPACE(qInitResources_kcachegrind)(); }
       ~initializer() { QT_RCC_MANGLE_NAMESPACE(qCleanupResources_kcachegrind)(); }
   } dummy;
}

#ifdef __clang__
#   pragma clang diagnostic pop
#endif
